// ------------------------------------------------------------
// Machinery project - math library
// ------------------------------------------------------------
// (c) 2000 by Sascha 'EvilOne' Salevsky
// E-mail: Sascha.Salevsky@Ibykus.de
// ------------------------------------------------------------

#ifndef _mathematics_h_
#error "Dont include Vector2 directly!!! Use Mathematics.h instead."
#endif

// Constructors

inline Vector2::Vector2()
{}

inline Vector2::Vector2(const float* other)
{
	x = other[0]; y = other[1];
}

inline Vector2::Vector2(float _x, float _y)
{
	x = _x; y = _y;
}

inline Vector2::Vector2(const Vector2& other)
{
	x = other.x; y = other.y;
}

// Assign

inline Vector2& Vector2::operator=(const Vector2& other)
{
	x = other.x; y = other.y;
	return *this;
}

inline void Vector2::Assign(float _x, float _y)
{
	x = _x; y = _y;
}


inline void Vector2::Assign(const Vector2& other)
{
	x = other.x; y = other.y;
}

inline void Vector2::Assign(float* other)
{
	x = other[0]; y = other[1];
}

// Swap

inline void Vector2::Swap(Vector2& other)
{
	Vector2 tmp = *this;
	Assign(other);
	other.Assign(tmp);
}

// Casting

inline Vector2::operator float*()
{
	return &x;
}

inline Vector2::operator const float*() const
{
	return &x;
}

// Math

inline void Vector2::Add(const Vector2& other)
{
	x += other.x; y += other.y;
}

inline void Vector2::Add(const Vector2& v1, const Vector2& v2)
{
	x = v1.x + v2.x; y = v1.y + v2.y;
}

inline void Vector2::Sub(const Vector2& other)
{
	x -= other.x; y -= other.y;
}

inline void Vector2::Sub(const Vector2& v1, const Vector2& v2)
{
	x = v1.x - v2.x; y = v1.y - v2.y;
}

inline void Vector2::Mul(float f)
{
	x *= f; y *= f;
}

inline void Vector2::Mul(const Vector2& other, float f)
{
	x = other.x * f; y = other.y * f;
}

inline void Vector2::Div(float f)
{
	f = 1.0f / f;
	x *= f; y *= f;
}

inline void Vector2::Div(const Vector2& other, float f)
{
	f = 1.0f / f;
	x = other.x * f; y = other.y * f;
}

inline void Vector2::Invert()
{
	x = -x; x = -y;
}

inline void Vector2::Invert(const Vector2& other)
{
	x = -other.x; x = -other.y;
}

// Linear combination

inline void Vector2::Combine(const Vector2& other, float s)
{
	x += other.x * s; y += other.y * s;
}

inline void Vector2::Combine(float s, const Vector2& other, float t)
{
	x = x * s + other.x * t; y = y * s + other.y * t;
}

inline void Vector2::Combine(const Vector2& v1, float s, const Vector2& v2, float t)
{
	x = v1.x * s + v2.x * t; y = v1.y * s + v2.y * t;
}

// operators

inline Vector2& Vector2::operator+=(const Vector2& other)
{
	x += other.x; y += other.y;
	return *this;
}

inline Vector2& Vector2::operator-=(const Vector2& other)
{
	x -= other.x; y -= other.y;
	return *this;
}

inline Vector2& Vector2::operator*=(float f)
{
	x *= f; y *= f;
	return *this;
}

inline Vector2& Vector2::operator-=(float f)
{
	x -= f; y -= f;
	return *this;
}

inline Vector2& Vector2::operator+=(float f)
{
	x += f; y += f;
	return *this;
}

inline Vector2& Vector2::operator/=(float f)
{
	f = 1.0f / f;
	x *= f; y *= f;
	return *this;
}

inline Vector2& Vector2::operator-()
{
	x = -x; y = -y;
	return *this;
}

inline Vector2 Vector2::operator+(const Vector2& other) const
{
	return Vector2(x + other.x, y + other.y);
}

inline Vector2 Vector2::operator-(const Vector2& other) const
{
	return Vector2(x - other.x, y - other.y);
}

inline Vector2 Vector2::operator+(float f) const
{
	return Vector2(x + f, y + f);
}

inline Vector2 Vector2::operator-(float f) const
{
	return Vector2(x - f, y - f);
}

inline Vector2 Vector2::operator*(float f) const
{
	return Vector2(x * f, y * f);
}

inline Vector2 Vector2::operator/(float f) const
{
	f = 1.0f / f;
	return Vector2(x * f, y * f);
}

// Linear algebra

inline float Vector2::Length() const
{
	return (float)sqrt(x * x + y * y);
}

inline float Vector2::LengthSqr() const
{
	return x * x + y * y;
}

inline float Vector2::Distance(const Vector2& other) const
{
	Vector2 tmp = other - *this;
	return tmp.Length();
}

inline float Vector2::DistanceSqr(const Vector2& other) const
{
	Vector2 tmp = other - *this;
	return tmp.LengthSqr();
}

inline void Vector2::Normalize()
{
	float f = 1.0f / Length();
	x = x * f; y = y * f;
}

inline void Vector2::Normalize(const Vector2& other)
{
	float f = 1.0f / other.Length();
	x = other.x * f; y = other.y * f;
}

inline void Vector2::Rotate(float angle)
{
	float _x = x;
	x =  _x * CosD(angle) + y * SinD(angle);
	y = -_x * SinD(angle) + y * CosD(angle);
}

inline void Vector2::Rotate(Vector2 other, float angle)
{
	float _x = other.x;
	float _y = other.y;

	x =  _x * CosD(angle) + _y * SinD(angle);
	y = -_x * SinD(angle) + _y * CosD(angle);
}

// Others

inline void Vector2::Clamp(float min, float max)
{
	x = ::Clamp(x, min, max); y = ::Clamp(y, min, max);
}

inline void Vector2::Lerp(const Vector2& v1, const Vector2& v2, float s)
{
	x = ::Lerp(s, v1.x, v2.x); y = ::Lerp(s, v1.y, v2.y);
}

inline void Vector2::Transform(const Matrix3& mat, const Vector2& other)
{
	const float* m = mat.M;
	float fx = other.x; float fy = other.y;
	x = m[0] * fx + m[3] * fy + m[6];  // w assumed to be 1.0F
	y = m[1] * fx + m[4] * fy + m[7];
}

inline void Vector2::Transform(const Matrix3& mat)
{
	const float* m = mat.M;
	float fx = x; float fy = y;
	x = m[0] * fx + m[3] * fy + m[6];  // w assumed to be 1.0F
	y = m[1] * fx + m[4] * fy + m[7];
}
